<?php
namespace AkTest\InputFilter;

use Ak\Filter\ArrayFilter;
use Ak\Validator\ValidatorPluginManager;
use Laminas\Filter\FilterChain;
use Laminas\Filter\FilterInterface;
use Laminas\Filter\FilterPluginManager;
use Laminas\InputFilter\Factory;
use Laminas\InputFilter\Input;
use Laminas\InputFilter\InputFilter;
use Laminas\Validator\NotEmpty;
use Laminas\Validator\ValidatorChain;

/**
 * Class InputFilterTest
 *
 * @package AkTest\InputFilter
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/input-filter
 */
final class InputFilterTest extends AbstractTestCase
{
    public function testCorrectPluginManagersAreInjected()
    {
        /** @var InputFilter $filter */
        $filter = $this->getInputFilter('inputfilter');
        $factory = $filter->getFactory();
        $container = $this->getContainer();

        $inputFilters = $this->getInputFilterManager();
        $this->assertSame($inputFilters, $factory->getInputFilterManager());

        /** @var FilterChain $filterChain */
        $filterChain = $factory->getDefaultFilterChain();
        $this->assertInstanceOf(FilterChain::class, $filterChain);

        $filters = $container->get(FilterPluginManager::class);
        $this->assertSame($filters, $filterChain->getPluginManager());

        /** @var ValidatorChain $validatorChain */
        $validatorChain = $factory->getDefaultValidatorChain();
        $this->assertInstanceOf(ValidatorChain::class, $factory->getDefaultValidatorChain());

        $validators = $container->get(ValidatorPluginManager::class);
        $this->assertSame($validators, $validatorChain->getPluginManager());
    }

    public function testCanCreateFromSpec()
    {
        $factory = new Factory($this->getInputFilterManager());
        $inner = $this->createMock(FilterInterface::class);

        $filter = $factory->createInputFilter([
            'field' => [
                'filters' => [[
                    'name' => 'array',
                    'options' => [
                        'filter' => $inner,
                    ],
                ]],
                'validators' => [[
                    'name' => 'notempty',
                ]],
            ],
        ]);

        $this->assertTrue($filter->has('field'));
        $input = $filter->get('field');

        $this->assertInstanceOf(Input::class, $input);
        $filters = $input->getFilterChain();
        $validators = $input->getValidatorChain();

        /** @var ArrayFilter $array */
        $array = $filters->getFilters()->top();
        $this->assertInstanceOf(ArrayFilter::class, $array);
        $this->assertSame($inner, $array->getFilter());

        $this->assertInstanceOf(NotEmpty::class, $validators->getValidators()[0]['instance']);
    }
}
